@@ -26,9 +26,6 @@ from barbican.queue import client as async_client
 
 LOG = utils.getLogger(__name__)
 
-_DEPRECATION_MSG = '%s has been deprecated in the Newton release. ' \
-                   'It will be removed in the Pike release.'
-
 
 def _order_not_found():
     """Throw exception indicating order not found."""
@@ -188,7 +185,7 @@ class OrdersController(controllers.ACLMixin):
         request_id = None
         ctxt = controllers._get_barbican_context(pecan.request)
         if ctxt:
-            new_order.creator_id = ctxt.user
+            new_order.creator_id = ctxt.user_id
             request_id = ctxt.request_id
 
         self.order_repo.create_from(new_order)
