@@ -21,11 +21,14 @@ quite intense for sqlalchemy, and maybe could be simplified.
 """
 
 import logging
+import re
 import sys
 import time
-import uuid
 
+from oslo_db import exception as db_exc
+from oslo_db.sqlalchemy import enginefacade
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import sqlalchemy
 from sqlalchemy import func as sa_func
 from sqlalchemy import or_
@@ -49,6 +52,7 @@ sa_logger = None
 # Singleton repository references, instantiated via get_xxxx_repository()
 #   functions below.  Please keep this list in alphabetical order.
 _CA_REPOSITORY = None
+_CONTAINER_ACL_USER_REPOSITORY = None
 _CONTAINER_ACL_REPOSITORY = None
 _CONTAINER_CONSUMER_REPOSITORY = None
 _CONTAINER_REPOSITORY = None
@@ -63,6 +67,7 @@ _PREFERRED_CA_REPOSITORY = None
 _PROJECT_REPOSITORY = None
 _PROJECT_CA_REPOSITORY = None
 _PROJECT_QUOTAS_REPOSITORY = None
+_SECRET_ACL_USER_REPOSITORY = None
 _SECRET_ACL_REPOSITORY = None
 _SECRET_META_REPOSITORY = None
 _SECRET_USER_META_REPOSITORY = None
@@ -70,6 +75,7 @@ _SECRET_REPOSITORY = None
 _TRANSPORT_KEY_REPOSITORY = None
 _SECRET_STORES_REPOSITORY = None
 _PROJECT_SECRET_STORE_REPOSITORY = None
+_SECRET_CONSUMER_REPOSITORY = None
 
 
 CONF = config.CONF
@@ -89,10 +95,10 @@ def hard_reset():
     setup_database_engine_and_factory()
 
 
-def setup_database_engine_and_factory():
+def setup_database_engine_and_factory(initialize_secret_stores=False):
     global sa_logger, _SESSION_FACTORY, _ENGINE
 
-    LOG.info(u._LI('Setting up database engine and session factory'))
+    LOG.info('Setting up database engine and session factory')
     if CONF.debug:
         sa_logger = logging.getLogger('sqlalchemy.engine')
         sa_logger.setLevel(logging.DEBUG)
@@ -106,7 +112,8 @@ def setup_database_engine_and_factory():
     # session instance per thread.
     session_maker = sa_orm.sessionmaker(bind=_ENGINE)
     _SESSION_FACTORY = sqlalchemy.orm.scoped_session(session_maker)
-    _initialize_secret_stores_data()
+    if initialize_secret_stores:
+        _initialize_secret_stores_data()
 
 
 def start():
@@ -160,33 +167,14 @@ def get_session():
 
 def _get_engine(engine):
     if not engine:
-        connection = CONF.sql_connection
-        if not connection:
-            raise exception.BarbicanException(
-                u._('No SQL connection configured'))
-
-    # TODO(jfwood):
-    # connection_dict = sqlalchemy.engine.url.make_url(_CONNECTION)
-
-        engine_args = {
-            'pool_recycle': CONF.sql_idle_timeout,
-            'echo': False,
-            'convert_unicode': True}
-        if CONF.sql_pool_class:
-            engine_args['poolclass'] = utils.get_class_for(
-                'sqlalchemy.pool', CONF.sql_pool_class)
-        if CONF.sql_pool_size:
-            engine_args['pool_size'] = CONF.sql_pool_size
-        if CONF.sql_pool_max_overflow:
-            engine_args['max_overflow'] = CONF.sql_pool_max_overflow
-
         db_connection = None
         try:
-            engine = _create_engine(connection, **engine_args)
+            engine = _create_engine()
             db_connection = engine.connect()
         except Exception as err:
-            msg = u._("Error configuring registry database with supplied "
-                      "sql_connection. Got error: {error}").format(error=err)
+            msg = u._(
+                "Error configuring registry database with supplied "
+                "database connection. Got error: {error}").format(error=err)
             LOG.exception(msg)
             raise exception.BarbicanException(msg)
         finally:
@@ -200,11 +188,22 @@ def _get_engine(engine):
 
             _auto_generate_tables(engine, tables)
         else:
-            LOG.info(u._LI('Not auto-creating barbican registry DB'))
+            LOG.info('Not auto-creating barbican registry DB')
 
     return engine
 
 
+def _create_engine():
+    return enginefacade.writer.get_engine()
+
+
+def model_query(model, *args, **kwargs):
+    """Query helper for simpler session usage."""
+    session = kwargs.get('session')
+    query = session.query(model, *args)
+    return query
+
+
 def _initialize_secret_stores_data():
     """Initializes secret stores data in database.
 
@@ -228,65 +227,20 @@ def is_db_connection_error(args):
     return False
 
 
-def _create_engine(connection, **engine_args):
-    LOG.debug(u._('Sql connection: please check "sql_connection" property in '
-                  'barbican configuration file; Args: %s'), engine_args)
-
-    engine = sqlalchemy.create_engine(connection, **engine_args)
-
-    # TODO(jfwood): if 'mysql' in connection_dict.drivername:
-    # TODO(jfwood): sqlalchemy.event.listen(_ENGINE, 'checkout',
-    # TODO(jfwood):                         ping_listener)
-
-    # Wrap the engine's connect method with a retry decorator.
-    engine.connect = wrap_db_error(engine.connect)
-
-    return engine
-
-
 def _auto_generate_tables(engine, tables):
     if tables and 'alembic_version' in tables:
         # Upgrade the database to the latest version.
-        LOG.info(u._LI('Updating schema to latest version'))
+        LOG.info('Updating schema to latest version')
         commands.upgrade()
     else:
         # Create database tables from our models.
-        LOG.info(u._LI('Auto-creating barbican registry DB'))
+        LOG.info('Auto-creating barbican registry DB')
         models.BASE.metadata.create_all(engine)
 
         # Sync the alembic version 'head' with current models.
         commands.stamp()
 
 
-def wrap_db_error(f):
-    """Retry DB connection. Copied from nova and modified."""
-    def _wrap(*args, **kwargs):
-        try:
-            return f(*args, **kwargs)
-        except sqlalchemy.exc.OperationalError as e:
-            if not is_db_connection_error(e.args[0]):
-                raise
-
-            remaining_attempts = CONF.sql_max_retries
-            while True:
-                LOG.warning(u._LW('SQL connection failed. %d attempts left.'),
-                            remaining_attempts)
-                remaining_attempts -= 1
-                time.sleep(CONF.sql_retry_interval)
-                try:
-                    return f(*args, **kwargs)
-                except sqlalchemy.exc.OperationalError as e:
-                    if (remaining_attempts <= 0 or not
-                            is_db_connection_error(e.args[0])):
-                        raise
-                except sqlalchemy.exc.DBAPIError:
-                    raise
-        except sqlalchemy.exc.DBAPIError:
-            raise
-    _wrap.__name__ = f.__name__
-    return _wrap
-
-
 def clean_paging_values(offset_arg=0, limit_arg=CONF.default_limit_paging):
     """Cleans and safely limits raw paging offset/limit values."""
     offset_arg = offset_arg or 0
@@ -310,7 +264,7 @@ def clean_paging_values(offset_arg=0, limit_arg=CONF.default_limit_paging):
     except ValueError:
         limit = CONF.default_limit_paging
 
-    LOG.debug(u._("Clean paging values limit=%(limit)s, offset=%(offset)s") %
+    LOG.debug("Clean paging values limit=%(limit)s, offset=%(offset)s" %
               {'limit': limit,
                'offset': offset})
 
@@ -373,7 +327,7 @@ class BaseRepo(object):
             entity = query.one()
 
         except sa_orm.exc.NoResultFound:
-            LOG.exception(u._LE("Not found for %s"), entity_id)
+            LOG.exception("Not found for %s", entity_id)
             entity = None
             if not suppress_exception:
                 _raise_entity_not_found(self._do_entity_name(), entity_id)
@@ -395,6 +349,7 @@ class BaseRepo(object):
             raise exception.Invalid(msg)
 
         LOG.debug("Begin create from...")
+        session = self.get_session(session)
         start = time.time()  # DEBUG
 
         # Validate the attributes before we go any further. From my
@@ -407,15 +362,34 @@ class BaseRepo(object):
         try:
             LOG.debug("Saving entity...")
             entity.save(session=session)
-        except sqlalchemy.exc.IntegrityError:
-            LOG.exception(u._LE('Problem saving entity for create'))
-            _raise_entity_already_exists(self._do_entity_name())
+        except db_exc.DBDuplicateEntry as e:
+            session.rollback()
+            LOG.exception('Problem saving entity for create')
+            error_msg = re.sub('[()]', '', str(e.args))
+            raise exception.ConstraintCheck(error=error_msg)
 
         LOG.debug('Elapsed repo '
                   'create secret:%s', (time.time() - start))  # DEBUG
 
         return entity
 
+    def update_from(self, model_class, entity_id, values, session=None):
+        if id in values:
+            raise Exception('Cannot update id')
+        LOG.debug('Begin update from ...')
+        session = self.get_session(session=session)
+
+        query = session.query(model_class)
+        query = query.filter_by(id=entity_id)
+        try:
+            LOG.debug('Updating value ...')
+            entity = query.one()
+        except sa_orm.exc.NoResultFound:
+            raise exception.NotFound('DB Entity with id {0} not '
+                                     'found'.format(entity_id))
+        self._update_values(entity, values)
+        entity.save()
+
     def save(self, entity):
         """Saves the state of the entity."""
         entity.updated_at = timeutils.utcnow()
@@ -559,8 +533,7 @@ class BaseRepo(object):
                 # Its a soft delete so its more like entity update
                 entity.delete(session=session)
         except sqlalchemy.exc.SQLAlchemyError:
-            LOG.exception(u._LE('Problem finding project related entity to '
-                                'delete'))
+            LOG.exception('Problem finding project related entity to delete')
             if not suppress_exception:
                 raise exception.BarbicanException(u._('Error deleting project '
                                                       'entities for '
@@ -596,7 +569,7 @@ class ProjectRepo(BaseRepo):
         except sa_orm.exc.NoResultFound:
             entity = None
             if not suppress_exception:
-                LOG.exception(u._LE("Problem getting Project %s"),
+                LOG.exception("Problem getting Project %s",
                               external_project_id)
                 raise exception.NotFound(u._(
                     "No {entity_name} found with keystone-ID {id}").format(
@@ -799,7 +772,7 @@ class SecretRepo(BaseRepo):
         except sa_orm.exc.NoResultFound:
             entity = None
             if not suppress_exception:
-                LOG.exception(u._LE("Problem getting secret %s"),
+                LOG.exception("Problem getting secret %s",
                               entity_id)
                 raise exception.NotFound(u._(
                     "No secret found with secret-ID {id}").format(
@@ -966,27 +939,43 @@ class KEKDatumRepo(BaseRepo):
 
         session = self.get_session(session)
 
-        # TODO(jfwood): Reverse this...attempt insert first, then get on fail.
-        try:
-            query = session.query(models.KEKDatum)
-            query = query.filter_by(project_id=project.id,
-                                    plugin_name=plugin_name,
-                                    active=True,
-                                    deleted=False)
+        query = session.query(models.KEKDatum)
+        query = query.filter_by(project_id=project.id,
+                                plugin_name=plugin_name,
+                                active=True,
+                                deleted=False)
 
-            kek_datum = query.one()
+        query = query.order_by(models.KEKDatum.created_at)
 
-        except sa_orm.exc.NoResultFound:
+        kek_datums = query.all()
 
+        if not kek_datums:
             kek_datum = models.KEKDatum()
 
             kek_datum.kek_label = "project-{0}-key-{1}".format(
-                project.external_id, uuid.uuid4())
+                project.external_id, uuidutils.generate_uuid())
             kek_datum.project_id = project.id
             kek_datum.plugin_name = plugin_name
             kek_datum.status = models.States.ACTIVE
 
             self.save(kek_datum)
+        else:
+            kek_datum = kek_datums.pop()
+
+            # (alee)  There should be only one active KEKDatum.
+            # Due to a race condition with many threads or
+            # many barbican processes, its possible to have
+            # multiple active KEKDatum.  The code below makes
+            # all the extra KEKDatum inactive
+            # See LP#1726378
+            for kd in kek_datums:
+                LOG.debug(
+                    "Multiple active KEKDatum found for %s."
+                    "Setting %s to be inactive.",
+                    project.external_id,
+                    kd.kek_label)
+                kd.active = False
+                self.save(kd)
 
         return kek_datum
 
@@ -1266,7 +1255,7 @@ class ContainerRepo(BaseRepo):
     """Repository for the Container entity."""
 
     def get_by_create_date(self, external_project_id, offset_arg=None,
-                           limit_arg=None, name_arg=None,
+                           limit_arg=None, name_arg=None, type_arg=None,
                            suppress_exception=False, session=None):
         """Returns a list of containers
 
@@ -1286,6 +1275,9 @@ class ContainerRepo(BaseRepo):
         if name_arg:
             query = query.filter(models.Container.name.like(name_arg))
 
+        if type_arg:
+            query = query.filter(models.Container.type == type_arg)
+
         query = query.join(models.Project, models.Container.project)
         query = query.filter(models.Project.external_id == external_project_id)
 
@@ -1339,8 +1331,7 @@ class ContainerRepo(BaseRepo):
         except sa_orm.exc.NoResultFound:
             entity = None
             if not suppress_exception:
-                LOG.exception(u._LE("Problem getting container %s"),
-                              entity_id)
+                LOG.exception("Problem getting container %s", entity_id)
                 raise exception.NotFound(u._(
                     "No container found with container-ID {id}").format(
                         entity_name=self._do_entity_name(),
@@ -1349,19 +1340,19 @@ class ContainerRepo(BaseRepo):
 
 
 class ContainerSecretRepo(BaseRepo):
-        """Repository for the ContainerSecret entity."""
-        def _do_entity_name(self):
-            """Sub-class hook: return entity name, such as for debugging."""
-            return "ContainerSecret"
+    """Repository for the ContainerSecret entity."""
+    def _do_entity_name(self):
+        """Sub-class hook: return entity name, such as for debugging."""
+        return "ContainerSecret"
 
-        def _do_build_get_query(self, entity_id, external_project_id, session):
-            """Sub-class hook: build a retrieve query."""
-            return session.query(models.ContainerSecret
-                                 ).filter_by(id=entity_id)
+    def _do_build_get_query(self, entity_id, external_project_id, session):
+        """Sub-class hook: build a retrieve query."""
+        return session.query(models.ContainerSecret
+                             ).filter_by(id=entity_id)
 
-        def _do_validate(self, values):
-            """Sub-class hook: validate values."""
-            pass
+    def _do_validate(self, values):
+        """Sub-class hook: validate values."""
+        pass
 
 
 class ContainerConsumerRepo(BaseRepo):
@@ -1430,13 +1421,13 @@ class ContainerConsumerRepo(BaseRepo):
             container.updated_at = timeutils.utcnow()
             container.consumers.append(new_consumer)
             container.save(session=session)
-        except sqlalchemy.exc.IntegrityError:
+        except db_exc.DBDuplicateEntry:
             session.rollback()  # We know consumer already exists.
 
             # This operation is idempotent, so log this and move on
-            LOG.debug("Consumer %s already exists for container %s,"
-                      " continuing...", (new_consumer.name, new_consumer.URL),
-                      new_consumer.container_id)
+            LOG.debug("Consumer %s with URL %s already exists for "
+                      "container %s, continuing...", new_consumer.name,
+                      new_consumer.URL, new_consumer.container_id)
             # Get the existing entry and reuse it by clearing the deleted flags
             existing_consumer = self.get_by_values(
                 new_consumer.container_id, new_consumer.name, new_consumer.URL,
@@ -1716,7 +1707,7 @@ class CertificateAuthorityMetadatumRepo(BaseRepo):
         except sa_orm.exc.NoResultFound:
             metadata = dict()
 
-        return dict((m.key, m.value) for m in metadata)
+        return {(m.key, m.value) for m in metadata}
 
     def _do_entity_name(self):
         """Sub-class hook: return entity name, such as for debugging."""
@@ -1984,6 +1975,26 @@ class SecretACLRepo(BaseRepo):
             entity.delete(session=session)
 
 
+class SecretACLUserRepo(BaseRepo):
+    """Repository for the SecretACLUser entity."""
+
+    def _do_entity_name(self):
+        """Sub-class hook: return entity name, such as for debugging."""
+        return "SecretACLUser"
+
+    def _do_build_get_query(self, entity_id, external_project_id, session):
+        """Sub-class hook: build a retrieve query."""
+
+        query = session.query(models.SecretACLUser)
+        query = query.filter_by(id=entity_id)
+
+        return query
+
+    def _do_validate(self, values):
+        """Sub-class hook: validate values."""
+        pass
+
+
 class ContainerACLRepo(BaseRepo):
     """Repository for the ContainerACL entity.
 
@@ -2080,6 +2091,25 @@ class ContainerACLRepo(BaseRepo):
             entity.delete(session=session)
 
 
+class ContainerACLUserRepo(BaseRepo):
+    """Repository for ContainerACLUser entity."""
+    def _do_entity_name(self):
+        """Sub-class hook: return entity name, such as for debugging."""
+        return "ContainerACLUser"
+
+    def _do_build_get_query(self, entity_id, external_project_id, session):
+        """Sub-class hook: build a retrieve query."""
+
+        query = session.query(models.ContainerACLUser)
+        query = query.filter_by(id=entity_id)
+
+        return query
+
+    def _do_validate(self, values):
+        """Sub-class hook: validate values."""
+        pass
+
+
 class ProjectQuotasRepo(BaseRepo):
     """Repository for the ProjectQuotas entity."""
     def _do_entity_name(self):
@@ -2276,7 +2306,7 @@ class ProjectSecretStoreRepo(BaseRepo):
         try:
             entity = query.one()
         except sa_orm.exc.NoResultFound:
-            LOG.info(u._LE("No preferred secret store found for project = %s"),
+            LOG.info("No preferred secret store found for project = %s",
                      project_id)
             entity = None
             if not suppress_exception:
@@ -2351,6 +2381,157 @@ class ProjectSecretStoreRepo(BaseRepo):
             project_id=project_id)
 
 
+class SecretConsumerRepo(BaseRepo):
+    """Repository for the SecretConsumer entity."""
+
+    def get_by_secret_id(self, secret_id,
+                         offset_arg=None, limit_arg=None,
+                         suppress_exception=False, session=None):
+        """Returns a list of SecretConsumers for a specific secret_id
+
+        The list is ordered by the date they were created at and paged
+        based on the offset and limit fields.
+        """
+
+        offset, limit = clean_paging_values(offset_arg, limit_arg)
+
+        session = self.get_session(session)
+
+        query = session.query(models.SecretConsumerMetadatum)
+        query = query.order_by(models.SecretConsumerMetadatum.created_at)
+        query = query.filter_by(deleted=False)
+        query = query.filter(
+            models.SecretConsumerMetadatum.secret_id == secret_id
+        )
+
+        start = offset
+        end = offset + limit
+        LOG.debug('Retrieving from %s to %s', start, end)
+        total = query.count()
+        entities = query.offset(start).limit(limit).all()
+        LOG.debug('Number entities retrieved: %s out of %s',
+                  len(entities), total
+                  )
+
+        if total <= 0 and not suppress_exception:
+            _raise_no_entities_found(self._do_entity_name())
+
+        return entities, offset, limit, total
+
+    def get_by_resource_id(self, resource_id,
+                           offset_arg=None, limit_arg=None,
+                           suppress_exception=False, session=None):
+        """Returns a list of SecretConsumers for a specific resource_id
+
+        The list is ordered by the date they were created at and paged
+        based on the offset and limit fields.
+        """
+
+        offset, limit = clean_paging_values(offset_arg, limit_arg)
+
+        session = self.get_session(session)
+
+        query = session.query(models.SecretConsumerMetadatum)
+        query = query.order_by(models.SecretConsumerMetadatum.created_at)
+        query = query.filter_by(deleted=False)
+        query = query.filter(
+            models.SecretConsumerMetadatum.resource_id == resource_id
+        )
+
+        start = offset
+        end = offset + limit
+        LOG.debug('Retrieving from %s to %s', start, end)
+        total = query.count()
+        entities = query.offset(start).limit(limit).all()
+        LOG.debug('Number entities retrieved: %s out of %s',
+                  len(entities), total
+                  )
+
+        if total <= 0 and not suppress_exception:
+            _raise_no_entities_found(self._do_entity_name())
+
+        return entities, offset, limit, total
+
+    def get_by_values(self, secret_id, service, resource_type, resource_id,
+                      suppress_exception=False,
+                      show_deleted=False, session=None):
+        session = self.get_session(session)
+
+        try:
+            query = session.query(models.SecretConsumerMetadatum)
+            query = query.filter_by(
+                secret_id=secret_id,
+                service=service,
+                resource_type=resource_type,
+                resource_id=resource_id,
+            )
+
+            if not show_deleted:
+                query.filter_by(deleted=False)
+            consumer = query.one()
+        except sa_orm.exc.NoResultFound:
+            consumer = None
+            if not suppress_exception:
+                raise exception.NotFound(
+                    u._("Could not find {entity_name}").format(
+                        entity_name=self._do_entity_name()))
+
+        return consumer
+
+    def create_or_update_from(self, new_consumer, secret, session=None):
+        session = self.get_session(session)
+        try:
+            secret.updated_at = timeutils.utcnow()
+            secret.consumers.append(new_consumer)
+            secret.save(session=session)
+        except db_exc.DBDuplicateEntry:
+            session.rollback()  # We know consumer already exists.
+
+            # This operation is idempotent, so log this and move on
+            LOG.debug(
+                "Consumer with resource_id %s already exists for secret %s...",
+                new_consumer.resource_id, new_consumer.secret_id
+            )
+            # Get the existing entry and reuse it by clearing the deleted flags
+            existing_consumer = self.get_by_values(
+                new_consumer.secret_id,
+                new_consumer.service,
+                new_consumer.resource_type,
+                new_consumer.resource_id,
+                show_deleted=True
+            )
+            existing_consumer.deleted = False
+            existing_consumer.deleted_at = None
+            # We are not concerned about timing here -- set only, no reads
+            existing_consumer.save()
+
+    def _do_entity_name(self):
+        """Sub-class hook: return entity name, such as for debugging."""
+        return "SecretConsumer"
+
+    def _do_build_get_query(self, entity_id, external_project_id, session):
+        """Sub-class hook: build a retrieve query."""
+        query = session.query(models.SecretConsumerMetadatum)
+        return query.filter_by(id=entity_id, deleted=False)
+
+    def _do_validate(self, values):
+        """Sub-class hook: validate values."""
+        pass
+
+    def _build_get_project_entities_query(self, project_id, session):
+        """Builds query for retrieving consumers associated with given project
+
+        :param project_id: id of barbican project entity
+        :param session: existing db session reference.
+        """
+        query = session.query(
+            models.SecretConsumerMetadatum).filter_by(deleted=False)
+        query = query.filter(
+            models.SecretConsumerMetadatum.project_id == project_id)
+
+        return query
+
+
 def get_ca_repository():
     """Returns a singleton Secret repository instance."""
     global _CA_REPOSITORY
@@ -2382,6 +2563,13 @@ def get_container_secret_repository():
     return _get_repository(_CONTAINER_SECRET_REPOSITORY, ContainerSecretRepo)
 
 
+def get_container_acl_user_repository():
+    """Returns a singleton Container-ACL-User repository instance."""
+    global _CONTAINER_ACL_USER_REPOSITORY
+    return _get_repository(_CONTAINER_ACL_USER_REPOSITORY,
+                           ContainerACLUserRepo)
+
+
 def get_encrypted_datum_repository():
     """Returns a singleton Encrypted Datum repository instance."""
     global _ENCRYPTED_DATUM_REPOSITORY
@@ -2453,6 +2641,12 @@ def get_secret_acl_repository():
     return _get_repository(_SECRET_ACL_REPOSITORY, SecretACLRepo)
 
 
+def get_secret_acl_user_repository():
+    """Returns a singleton Secret-ACL-User repository instance."""
+    global _SECRET_ACL_USER_REPOSITORY
+    return _get_repository(_SECRET_ACL_USER_REPOSITORY, SecretACLUserRepo)
+
+
 def get_secret_meta_repository():
     """Returns a singleton Secret meta repository instance."""
     global _SECRET_META_REPOSITORY
@@ -2491,6 +2685,13 @@ def get_project_secret_store_repository():
                            ProjectSecretStoreRepo)
 
 
+def get_secret_consumer_repository():
+    """Returns a singleton Secret Consumer repository instance."""
+    global _SECRET_CONSUMER_REPOSITORY
+    return _get_repository(_SECRET_CONSUMER_REPOSITORY,
+                           SecretConsumerRepo)
+
+
 def _get_repository(global_ref, repo_class):
     if not global_ref:
         global_ref = repo_class()
@@ -2512,9 +2713,3 @@ def _raise_no_entities_found(entity_name):
     raise exception.NotFound(
         u._("No entities of type {entity_name} found").format(
             entity_name=entity_name))
-
-
-def _raise_entity_already_exists(entity_name):
-    raise exception.Duplicate(
-        u._("Entity '{entity_name}' "
-            "already exists").format(entity_name=entity_name))
