@@ -19,7 +19,6 @@ from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.primitives import serialization
 from oslo_config import cfg
 from oslo_utils import encodeutils
-import six
 
 from barbican.common import config
 from barbican.common import utils
@@ -33,10 +32,13 @@ LOG = utils.getLogger(__name__)
 simple_crypto_plugin_group = cfg.OptGroup(name='simple_crypto_plugin',
                                           title="Simple Crypto Plugin Options")
 simple_crypto_plugin_opts = [
-    cfg.StrOpt('kek',
-               default='dGhpcnR5X3R3b19ieXRlX2tleWJsYWhibGFoYmxhaGg=',
-               help=u._('Key encryption key to be used by Simple Crypto '
-                        'Plugin'), secret=True),
+    cfg.MultiStrOpt(
+        'kek',
+        default=['dGhpcnR5X3R3b19ieXRlX2tleWJsYWhibGFoYmxhaGg='],
+        secret=True,
+        help=u._('Fernet Key-Encryption Key (KEK) to be used by SimpleCrypto '
+                 'Plugin to encrypt Project-specific KEKs.'),
+    ),
     cfg.StrOpt('plugin_name',
                help=u._('User friendly plugin name'),
                default='Software Only Crypto'),
@@ -54,7 +56,9 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
     """Insecure implementation of the crypto plugin."""
 
     def __init__(self, conf=CONF):
-        self.master_kek = conf.simple_crypto_plugin.kek
+        if len(conf.simple_crypto_plugin.kek) < 1:
+            raise ValueError(u._("SimpleCrypto KEK is undefined"))
+        self.master_keys = conf.simple_crypto_plugin.kek
         self.plugin_name = conf.simple_crypto_plugin.plugin_name
         LOG.info("{} initialized".format(self.plugin_name))
 
@@ -65,9 +69,11 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
         if not kek_meta_dto.plugin_meta:
             raise ValueError(u._('KEK not yet created.'))
         # the kek is stored encrypted. Need to decrypt.
-        encryptor = fernet.Fernet(self.master_kek)
+        encryptor = fernet.MultiFernet(
+            [fernet.Fernet(x) for x in self.master_keys]
+        )
         # Note : If plugin_meta type is unicode, encode to byte.
-        if isinstance(kek_meta_dto.plugin_meta, six.text_type):
+        if isinstance(kek_meta_dto.plugin_meta, str):
             kek_meta_dto.plugin_meta = kek_meta_dto.plugin_meta.encode('utf-8')
 
         return encryptor.decrypt(kek_meta_dto.plugin_meta)
@@ -75,7 +81,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
     def encrypt(self, encrypt_dto, kek_meta_dto, project_id):
         kek = self._get_kek(kek_meta_dto)
         unencrypted = encrypt_dto.unencrypted
-        if not isinstance(unencrypted, six.binary_type):
+        if not isinstance(unencrypted, bytes):
             raise ValueError(
                 u._(
                     'Unencrypted data must be a byte type, but was '
@@ -101,7 +107,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
         kek_meta_dto.mode = 'cbc'
         if not kek_meta_dto.plugin_meta:
             # the kek is stored encrypted in the plugin_meta field
-            encryptor = fernet.Fernet(self.master_kek)
+            encryptor = fernet.Fernet(self.master_keys[0])
             key = fernet.Fernet.generate_key()
             kek_meta_dto.plugin_meta = encryptor.encrypt(key)
         return kek_meta_dto
@@ -122,7 +128,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
         - DSA, without passphrase (supported)
         - DSA, with passphrase (supported)
         """
-        if(generate_dto.algorithm is None or generate_dto
+        if (generate_dto.algorithm is None or generate_dto
                 .algorithm.lower() == 'rsa'):
             private_key = rsa.generate_private_key(
                 public_exponent=65537,
@@ -174,7 +180,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
 
         passphrase_dto = None
         if generate_dto.passphrase:
-            if isinstance(generate_dto.passphrase, six.text_type):
+            if isinstance(generate_dto.passphrase, str):
                 generate_dto.passphrase = generate_dto.passphrase.encode(
                     'utf-8')
 
@@ -189,8 +195,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
                  mode=None):
         if type_enum == c.PluginSupportTypes.ENCRYPT_DECRYPT:
             return True
-
-        if type_enum == c.PluginSupportTypes.SYMMETRIC_KEY_GENERATION:
+        elif type_enum == c.PluginSupportTypes.SYMMETRIC_KEY_GENERATION:
             return self._is_algorithm_supported(algorithm,
                                                 bit_length,
                                                 mode)
