@@ -14,15 +14,13 @@
 # limitations under the License.
 
 import datetime
-import six
 import unittest
 
 from oslo_serialization import base64
-import testtools
+from oslo_utils import timeutils
 
 from barbican.common import exception as excep
 from barbican.common import validators
-from barbican.tests import certificate_utils as certs
 from barbican.tests import keys
 from barbican.tests import utils
 
@@ -103,9 +101,9 @@ class WhenTestingValidatorsFunctions(utils.BaseTestCase):
         self.assertTrue(is_too_big)
 
     def test_secret_too_big_is_true_for_big_unicode_secrets(self):
-        beer = u'\U0001F37A'
+        beer = '\U0001F37A'
         data = beer * (validators.CONF.max_allowed_secret_in_bytes // 4)
-        data += u'1'
+        data += '1'
 
         is_too_big = validators.secret_too_big(data)
 
@@ -242,7 +240,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', six.text_type(exception))
+        self.assertIn('bit_length', str(exception))
 
     def test_should_raise_non_integer_bit_length(self):
         self.secret_req['bit_length'] = "23"
@@ -253,7 +251,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', six.text_type(exception))
+        self.assertIn('bit_length', str(exception))
 
     def test_should_raise_bit_length_less_than_min(self):
         self.secret_req['bit_length'] = 0
@@ -264,7 +262,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', six.text_type(exception))
+        self.assertIn('bit_length', str(exception))
 
     def test_should_raise_bit_length_greater_than_max(self):
         self.secret_req['bit_length'] = 32768
@@ -275,7 +273,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', six.text_type(exception))
+        self.assertIn('bit_length', str(exception))
 
     def test_should_raise_mode_length_greater_than_max(self):
         self.secret_req['mode'] = 'a' * 256
@@ -286,7 +284,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('mode', exception.invalid_property)
-        self.assertIn('mode', six.text_type(exception))
+        self.assertIn('mode', str(exception))
 
     def test_should_raise_mode_is_non_string(self):
         self.secret_req['mode'] = 123
@@ -297,7 +295,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('mode', exception.invalid_property)
-        self.assertIn('mode', six.text_type(exception))
+        self.assertIn('mode', str(exception))
 
     def test_validation_should_raise_with_empty_payload(self):
         self.secret_req['payload'] = '   '
@@ -308,7 +306,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('payload', exception.invalid_property)
-        self.assertIn('payload', six.text_type(exception))
+        self.assertIn('payload', str(exception))
 
     def test_should_raise_already_expired(self):
         self.secret_req['expiration'] = '2004-02-28T19:14:44.180394'
@@ -319,7 +317,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', six.text_type(exception))
+        self.assertIn('expiration', str(exception))
 
     def test_should_raise_expiration_nonsense(self):
         self.secret_req['expiration'] = 'nonsense'
@@ -330,7 +328,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', six.text_type(exception))
+        self.assertIn('expiration', str(exception))
 
     def test_should_raise_expiration_is_non_string(self):
         self.secret_req['expiration'] = 123
@@ -341,7 +339,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', six.text_type(exception))
+        self.assertIn('expiration', str(exception))
 
     def test_should_raise_expiration_greater_than_max(self):
         self.secret_req['expiration'] = 'a' * 256
@@ -352,7 +350,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', six.text_type(exception))
+        self.assertIn('expiration', str(exception))
 
     def test_should_raise_algorithm_is_non_string(self):
         self.secret_req['algorithm'] = 123
@@ -363,7 +361,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('algorithm', exception.invalid_property)
-        self.assertIn('algorithm', six.text_type(exception))
+        self.assertIn('algorithm', str(exception))
 
     def test_should_raise_algorithm_greater_than_max(self):
         self.secret_req['algorithm'] = 'a' * 256
@@ -374,7 +372,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('algorithm', exception.invalid_property)
-        self.assertIn('algorithm', six.text_type(exception))
+        self.assertIn('algorithm', str(exception))
 
     def test_should_raise_all_nulls(self):
         self.secret_req = {'name': None,
@@ -416,7 +414,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.validator.validate(self.secret_req)
         except excep.InvalidObject as e:
             self.assertIsNotNone(e)
-            self.assertIsNotNone(six.text_type(e))
+            self.assertIsNotNone(str(e))
         else:
             self.fail('No validation exception was raised')
 
@@ -454,7 +452,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('payload_content_type', exception.invalid_property)
-        self.assertIn('payload_content_type', six.text_type(exception))
+        self.assertIn('payload_content_type', str(exception))
 
     def test_should_raise_with_payload_content_encoding_greater_than_max(self):
         self.secret_req['payload_content_encoding'] = 'a' * 256
@@ -465,7 +463,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
         )
         self.assertEqual('payload_content_encoding',
                          exception.invalid_property)
-        self.assertIn('payload_content_encoding', six.text_type(exception))
+        self.assertIn('payload_content_encoding', str(exception))
 
     def test_should_raise_with_plain_text_and_encoding(self):
         self.secret_req['payload_content_encoding'] = 'base64'
@@ -515,7 +513,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
     def test_validation_should_raise_with_unicode_payload(self):
         self.secret_req['payload_content_type'] = 'application/octet-stream'
         self.secret_req['payload_content_encoding'] = 'base64'
-        self.secret_req['payload'] = six.unichr(0x0080)
+        self.secret_req['payload'] = chr(0x0080)
 
         exception = self.assertRaises(
             excep.InvalidObject,
@@ -624,7 +622,7 @@ class WhenTestingContainerValidator(utils.BaseTestCase):
             self.container_req,
         )
         self.assertEqual('name', exception.invalid_property)
-        self.assertIn('name', six.text_type(exception))
+        self.assertIn('name', str(exception))
 
     def test_should_raise_nonstring_secret_name(self):
         self.secret_refs[0]["name"] = 5
@@ -668,7 +666,7 @@ class WhenTestingContainerValidator(utils.BaseTestCase):
             self.container_req,
         )
 
-        self.assertIn('type', six.text_type(exception))
+        self.assertIn('type', str(exception))
 
     def test_should_raise_empty_type(self):
         self.container_req['type'] = ''
@@ -1192,11 +1190,6 @@ class WhenTestingKeyTypeOrderValidator(utils.BaseTestCase):
 
         self.validator = validators.TypeOrderValidator()
 
-    def test_should_pass_with_certificate_type_in_order_refs(self):
-        self.key_order_req['type'] = 'certificate'
-        result = self.validator.validate(self.key_order_req)
-        self.assertEqual('certificate', result['type'])
-
     def test_should_pass_with_null_content_type_in_meta(self):
         self.key_order_req['meta']['payload_content_type'] = None
         result = self.validator.validate(self.key_order_req)
@@ -1211,7 +1204,7 @@ class WhenTestingKeyTypeOrderValidator(utils.BaseTestCase):
     def test_should_pass_good_exp_meta_in_order_refs(self):
         self.key_order_req['meta']['algorithm'] = 'AES'
         ony_year_factor = datetime.timedelta(days=1 * 365)
-        date_after_year = datetime.datetime.utcnow() + ony_year_factor
+        date_after_year = timeutils.utcnow() + ony_year_factor
         date_after_year_str = date_after_year.strftime('%Y-%m-%d %H:%M:%S')
         self.key_order_req['meta']['expiration'] = date_after_year_str
         result = self.validator.validate(self.key_order_req)
@@ -1258,7 +1251,7 @@ class WhenTestingKeyTypeOrderValidator(utils.BaseTestCase):
                                       self.validator.validate,
                                       self.key_order_req)
         self.assertIn("bit_length' is required field for key type order",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_with_zero_bit_length_in_order_refs(self):
         self.key_order_req['meta']['bit_length'] = 0
@@ -1347,7 +1340,7 @@ class WhenTestingAsymmetricTypeOrderValidator(utils.BaseTestCase):
                                       self.asymmetric_order_req)
         self.assertIn(
             "bit_length' is required field for asymmetric key type order",
-            six.text_type(exception))
+            str(exception))
 
     def test_should_raise_with_zero_bit_length_in_asymmetric_order_refs(self):
         self.asymmetric_order_req['meta']['bit_length'] = 0
@@ -1366,238 +1359,6 @@ class WhenTestingAsymmetricTypeOrderValidator(utils.BaseTestCase):
         self.assertEqual("bit_length", exception.invalid_property)
 
 
-class WhenTestingSimpleCMCOrderValidator(utils.BaseTestCase):
-
-    def setUp(self):
-        super(WhenTestingSimpleCMCOrderValidator, self).setUp()
-        self.type = 'certificate'
-        request_data = base64.encode_as_text(certs.create_good_csr())
-        self.meta = {'request_type': 'simple-cmc',
-                     'request_data': request_data,
-                     'requestor_name': 'Barbican User',
-                     'requestor_email': 'barbican_user@example.com',
-                     'requestor_phone': '555-1212'}
-        self._set_order()
-        self.validator = validators.TypeOrderValidator()
-
-    def _set_order(self):
-        self.order_req = {'type': self.type,
-                          'meta': self.meta}
-
-    def test_should_pass_good_data(self):
-        self.validator.validate(self.order_req)
-
-    def test_should_raise_with_no_metadata(self):
-        self.order_req = {'type': self.type}
-        self.assertRaises(excep.InvalidObject,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_bad_request_type(self):
-        self.meta['request_type'] = 'bad_request_type'
-        self._set_order()
-        self.assertRaises(excep.InvalidCertificateRequestType,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_no_request_data(self):
-        del self.meta['request_data']
-        self._set_order()
-        self.assertRaises(excep.MissingMetadataField,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_pkcs10_data_with_bad_base64(self):
-        self.meta['request_data'] = certs.create_bad_csr()
-        self._set_order()
-        self.assertRaises(excep.PayloadDecodingError,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_bad_pkcs10_data(self):
-        request_data = base64.encode_as_text(certs.create_bad_csr())
-        self.meta['request_data'] = request_data
-        self._set_order()
-        self.assertRaises(excep.InvalidPKCS10Data,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_signed_wrong_key_pkcs10_data(self):
-        self.meta['request_data'] = base64.encode_as_text(
-            certs.create_csr_signed_with_wrong_key())
-        self._set_order()
-        self.assertRaises(excep.InvalidPKCS10Data,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_unsigned_pkcs10_data(self):
-        self.meta['request_data'] = base64.encode_as_text(
-            certs.create_csr_that_has_not_been_signed())
-        self._set_order()
-        self.assertRaises(excep.InvalidPKCS10Data,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_payload_in_order(self):
-        self.meta['payload'] = 'payload'
-        self.assertRaises(excep.InvalidObject,
-                          self.validator.validate,
-                          self.order_req)
-
-
-class WhenTestingFullCMCOrderValidator(utils.BaseTestCase):
-
-    def setUp(self):
-        super(WhenTestingFullCMCOrderValidator, self).setUp()
-        self.type = 'certificate'
-        self.meta = {'request_type': 'full-cmc',
-                     'request_data': VALID_FULL_CMC,
-                     'requestor_name': 'Barbican User',
-                     'requestor_email': 'barbican_user@example.com',
-                     'requestor_phone': '555-1212'}
-        self._set_order()
-        self.validator = validators.TypeOrderValidator()
-
-    def _set_order(self):
-        self.order_req = {'type': self.type,
-                          'meta': self.meta}
-
-    def test_should_raise_not_yet_implemented(self):
-        self.assertRaises(excep.FullCMCNotSupported,
-                          self.validator.validate,
-                          self.order_req)
-
-    @testtools.skip("Feature not yet implemented")
-    def test_should_pass_good_data(self):
-        self.validator.validate(self.order_req)
-
-    @testtools.skip("Feature not yet implemented")
-    def test_should_raise_with_no_request_data(self):
-        del self.meta['request_data']
-        self._set_order()
-        self.assertRaises(excep.MissingMetadataField,
-                          self.validator.validate,
-                          self.order_req)
-
-    @testtools.skip("Not yet implemented")
-    def test_should_raise_with_bad_cmc_data(self):
-        self.meta['request_data'] = 'Bad CMC Data'
-        self._set_order()
-        self.assertRaises(excep.InvalidCMCData,
-                          self.validator.validate,
-                          self.order_req)
-
-
-class WhenTestingCustomOrderValidator(utils.BaseTestCase):
-
-    def setUp(self):
-        super(WhenTestingCustomOrderValidator, self).setUp()
-        self.type = 'certificate'
-        self.meta = {'request_type': 'custom',
-                     'ca_param_1': 'value_1',
-                     'ca_param_2': 'value_2',
-                     'requestor_name': 'Barbican User',
-                     'requestor_email': 'barbican_user@example.com',
-                     'requestor_phone': '555-1212'}
-        self._set_order()
-        self.validator = validators.TypeOrderValidator()
-
-    def _set_order(self):
-        self.order_req = {'type': self.type,
-                          'meta': self.meta}
-
-    def test_should_pass_good_data(self):
-        self.validator.validate(self.order_req)
-
-    def test_should_pass_with_no_request_type(self):
-        # defaults to custom
-        del self.meta['request_type']
-        self._set_order()
-        self.validator.validate(self.order_req)
-
-
-class WhenTestingStoredKeyOrderValidator(utils.BaseTestCase):
-
-    def setUp(self):
-        super(WhenTestingStoredKeyOrderValidator, self).setUp()
-        self.type = 'certificate'
-        self.meta = {'request_type': 'stored-key',
-                     'container_ref':
-                         'https://localhost/v1/containers/good_container_ref',
-                     'subject_dn': 'cn=barbican-server,o=example.com',
-                     'requestor_name': 'Barbican User',
-                     'requestor_email': 'barbican_user@example.com',
-                     'requestor_phone': '555-1212'}
-        self.order_req = {'type': self.type,
-                          'meta': self.meta}
-        self.validator = validators.TypeOrderValidator()
-
-    def test_should_pass_good_data(self):
-        self.validator.validate(self.order_req)
-
-    def test_should_raise_with_no_container_ref(self):
-        del self.meta['container_ref']
-        self.assertRaises(excep.MissingMetadataField,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_no_subject_dn(self):
-        del self.meta['subject_dn']
-        self.assertRaises(excep.MissingMetadataField,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_pass_with_profile_and_ca_id(self):
-        self.meta['ca_id'] = 'my_ca_id'
-        self.meta['profile'] = 'my_profile'
-        self.validator.validate(self.order_req)
-
-    def test_should_raise_with_profile_and_no_ca_id(self):
-        self.meta['profile'] = 'my_profile'
-        self.assertRaises(excep.MissingMetadataField,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_extensions_data(self):
-        self.meta['extensions'] = VALID_EXTENSIONS
-        self.assertRaises(excep.CertificateExtensionsNotSupported,
-                          self.validator.validate,
-                          self.order_req)
-
-    @testtools.skip("Not yet implemented")
-    def test_should_raise_with_bad_extensions_data(self):
-        self.meta['extensions'] = 'Bad extensions data'
-        self.assertRaises(excep.InvalidExtensionsData,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_pass_with_one_cn_in_dn(self):
-        self.meta['subject_dn'] = "CN=example1"
-        self.validator.validate(self.order_req)
-
-    def test_should_pass_with_two_cn_in_dn(self):
-        self.meta['subject_dn'] = "CN=example1,CN=example2"
-        self.validator.validate(self.order_req)
-
-    def test_should_raise_with_blank_dn(self):
-        self.meta['subject_dn'] = ""
-        self.assertRaises(excep.InvalidSubjectDN,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_bad_subject_dn(self):
-        self.meta['subject_dn'] = "Bad subject DN data"
-        self.assertRaises(excep.InvalidSubjectDN,
-                          self.validator.validate,
-                          self.order_req)
-
-    def test_should_raise_with_payload_in_order(self):
-        self.meta['payload'] = 'payload'
-        self.assertRaises(excep.InvalidObject,
-                          self.validator.validate,
-                          self.order_req)
-
-
 @utils.parameterized_test_case
 class WhenTestingAclValidator(utils.BaseTestCase):
     def setUp(self):
@@ -1814,7 +1575,7 @@ class WhenTestingSecretMetadataValidator(utils.BaseTestCase):
                                       self.validator.validate,
                                       self.metadata_req)
         self.assertIn("metadata' is a required property",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_key_non_string(self):
         self.key1 = 0
@@ -1827,7 +1588,7 @@ class WhenTestingSecretMetadataValidator(utils.BaseTestCase):
                                       self.validator.validate,
                                       metadata_req)
         self.assertIn("Invalid Metadata. Keys and Values must be Strings.",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_key_non_url_safe_string(self):
         self.key1 = "key/01"
@@ -1840,7 +1601,7 @@ class WhenTestingSecretMetadataValidator(utils.BaseTestCase):
                                       self.validator.validate,
                                       metadata_req)
         self.assertIn("Invalid Key. Key must be URL safe.",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_value_non_string(self):
         self.value1 = 0
@@ -1853,7 +1614,7 @@ class WhenTestingSecretMetadataValidator(utils.BaseTestCase):
                                       self.validator.validate,
                                       metadata_req)
         self.assertIn("Invalid Metadata. Keys and Values must be Strings.",
-                      six.text_type(exception))
+                      str(exception))
 
 
 @utils.parameterized_test_case
@@ -1896,7 +1657,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       self.metadata_req)
         self.assertIn("Provided object does not match schema "
                       "'SecretMetadatum'",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_key_no_key(self):
         del self.metadata_req[self.key2]
@@ -1905,7 +1666,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       self.metadata_req)
         self.assertIn("Provided object does not match schema "
                       "'SecretMetadatum'",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_key_no_value(self):
         del self.metadata_req[self.key1]
@@ -1914,7 +1675,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       self.metadata_req)
         self.assertIn("Provided object does not match schema "
                       "'SecretMetadatum'",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_key_non_string(self):
         self.value1 = 0
@@ -1928,7 +1689,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       metadata_req)
         self.assertIn("Provided object does not match schema "
                       "'SecretMetadatum'",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_key_non_url_safe_string(self):
         self.value1 = "key/01"
@@ -1941,7 +1702,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       self.validator.validate,
                                       metadata_req)
         self.assertIn("Invalid Key. Key must be URL safe.",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_value_non_string(self):
         self.value2 = 0
@@ -1955,7 +1716,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       metadata_req)
         self.assertIn("Provided object does not match schema "
                       "'SecretMetadatum'",
-                      six.text_type(exception))
+                      str(exception))
 
     def test_should_raise_invalid_extra_sent_key(self):
         self.value2 = 0
@@ -1970,7 +1731,7 @@ class WhenTestingSecretMetadatumValidator(utils.BaseTestCase):
                                       metadata_req)
         self.assertIn("Provided object does not match schema "
                       "'SecretMetadatum'",
-                      six.text_type(exception))
+                      str(exception))
 
 
 class WhenTestingSecretConsumerValidator(utils.BaseTestCase):
