@@ -19,7 +19,6 @@ from unittest import mock
 from cryptography import fernet
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization
-import six
 
 from barbican.model import models
 from barbican.plugin.crypto import base as plugin
@@ -42,7 +41,7 @@ class WhenTestingSimpleCryptoPlugin(utils.BaseTestCase):
         return self.plugin.bind_kek_metadata(kek_meta_dto)
 
     def test_encrypt_unicode_raises_value_error(self):
-        unencrypted = u'unicode_beer\U0001F37A'
+        unencrypted = 'unicode_beer\U0001F37A'
         encrypt_dto = plugin.EncryptDTO(unencrypted)
         secret = mock.MagicMock()
         secret.mime_type = 'text/plain'
@@ -66,9 +65,9 @@ class WhenTestingSimpleCryptoPlugin(utils.BaseTestCase):
                 Compare with unencrypted
         """
         project_kek = fernet.Fernet.generate_key()
-        encryptor = fernet.Fernet(self.plugin.master_kek)
+        encryptor = fernet.Fernet(self.plugin.master_keys[0])
         ENC_project_kek = encryptor.encrypt(project_kek)
-        UENC_project_kek = six.u(ENC_project_kek)
+        UENC_project_kek = ENC_project_kek
         kek_meta_dto = self._get_mocked_kek_meta_dto()
         kek_meta_dto.plugin_meta = UENC_project_kek
 
