@@ -13,10 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import base64
+import builtins
+import collections
+import os
 from unittest import mock
 
-import six
-
 from barbican.common import exception as ex
 from barbican.model import models
 from barbican.plugin.crypto import base as plugin_import
@@ -25,6 +27,11 @@ from barbican.plugin.crypto import pkcs11
 from barbican.tests import utils
 
 
+FakeKEKMetaDTO = collections.namedtuple(
+    'FakeKEKMetaDTO', 'kek_label, plugin_meta'
+)
+
+
 def generate_random_effect(length, session):
     return b'0' * length
 
@@ -42,7 +49,10 @@ class WhenTestingP11CryptoPlugin(utils.BaseTestCase):
         self.pkcs11.encrypt.return_value = {'iv': b'0', 'ct': b'0'}
         self.pkcs11.decrypt.return_value = b'0'
         self.pkcs11.generate_key.return_value = int(3)
-        self.pkcs11.wrap_key.return_value = {'iv': b'1', 'wrapped_key': b'1'}
+        self.pkcs11.wrap_key.return_value = {
+            'iv': b'1',
+            'wrapped_key': b'1',
+            'key_wrap_mechanism': 'CKM_AES_CBC_PAD'}
         self.pkcs11.unwrap_key.return_value = int(4)
         self.pkcs11.compute_hmac.return_value = b'1'
         self.pkcs11.verify_hmac.return_value = None
@@ -64,8 +74,11 @@ class WhenTestingP11CryptoPlugin(utils.BaseTestCase):
         self.cfg_mock.p11_crypto_plugin.encryption_mechanism = 'CKM_AES_CBC'
         self.cfg_mock.p11_crypto_plugin.seed_file = ''
         self.cfg_mock.p11_crypto_plugin.seed_length = 32
-        self.cfg_mock.p11_crypto_plugin.hmac_keywrap_mechanism = \
+        self.cfg_mock.p11_crypto_plugin.hmac_mechanism = \
             'CKM_SHA256_HMAC'
+        self.cfg_mock.p11_crypto_plugin.key_wrap_mechanism = \
+            'CKM_AES_CBC_PAD'
+        self.cfg_mock.p11_crypto_plugin.key_wrap_gen_iv = True
 
         self.plugin_name = 'Test PKCS11 plugin'
         self.cfg_mock.p11_crypto_plugin.plugin_name = self.plugin_name
@@ -158,7 +171,9 @@ class WhenTestingP11CryptoPlugin(utils.BaseTestCase):
 
     def test_decrypt(self):
         ct = b'ctct'
-        kek_meta_extended = '{"iv":"AAAA","mechanism":"CKM_AES_CBC"}'
+        kek_meta_extended = ('{"iv":"AAAA",'
+                             '"mechanism":"CKM_AES_CBC",'
+                             '"key_wrap_mechanism":"CKM_AES_CBC_PAD"}')
         decrypt_dto = plugin_import.DecryptDTO(ct)
         kek_meta = mock.MagicMock()
         kek_meta.kek_label = 'pkek'
@@ -309,7 +324,7 @@ class WhenTestingP11CryptoPlugin(utils.BaseTestCase):
         d = '01234567' * 4
         mo = mock.mock_open(read_data=d)
 
-        with mock.patch(six.moves.builtins.__name__ + '.open',
+        with mock.patch(builtins.__name__ + '.open',
                         mo,
                         create=True):
             p11 = self.plugin._create_pkcs11(ffi)
@@ -356,3 +371,45 @@ class WhenTestingP11CryptoPlugin(utils.BaseTestCase):
 
     def test_get_plugin_name(self):
         self.assertEqual(self.plugin_name, self.plugin.get_plugin_name())
+
+    def test_load_kek_from_meta_dto_no_key_wrap_mechanism(self):
+        key = base64.b64encode(os.urandom(32)).decode('UTF-8')
+        hmac = base64.b64encode(os.urandom(16)).decode('UTF-8')
+        fake_dto = FakeKEKMetaDTO('test_kek', p11_crypto.json_dumps_compact({
+            'iv': None,
+            'wrapped_key': key,
+            'hmac': hmac,
+            'mkek_label': 'test_mkek',
+            'hmac_label': 'test_hmac'
+        }))
+        load_mock = mock.MagicMock()
+        self.plugin._load_kek = load_mock
+
+        self.plugin._load_kek_from_meta_dto(fake_dto)
+
+        # key_wrap_mechanism should default to 'CKM_AES_CBC_PAD'
+        load_mock.assert_called_with(
+            'test_kek', None, key, hmac,
+            'test_mkek', 'test_hmac', 'CKM_AES_CBC_PAD')
+
+    def test_load_kek_no_iv(self):
+        key = os.urandom(32)
+        wrapped = base64.b64encode(key).decode('UTF-8')
+        hmac = base64.b64encode(os.urandom(16)).decode('UTF-8')
+
+        self.plugin._load_kek('test_key', None, wrapped, hmac, 'mkek_label',
+                              'hmac_label', 'CKM_AES_KEY_WRAP_KWP')
+
+        key in self.pkcs11.verify_hmac.call_args.args
+
+    def test_generate_wrapped_kek_no_iv(self):
+        wrapped = base64.b64encode(os.urandom(32))
+        self.pkcs11.wrap_key.return_value = {
+            'iv': None,
+            'wrapped_key': wrapped,
+            'key_wrap_mechanism': 'CKM_AES_KEY_WRAP_KWP'
+        }
+
+        _ = self.plugin._generate_wrapped_kek(32, 'test_kek')
+
+        wrapped in self.pkcs11.compute_hmac.call_args.args
